@@ -19,6 +19,15 @@ def test_simple_call():
     assert increment(increment(1)).call() == 3
 
 
+def test_simple_call():
+    @pfp.lazy
+    def increment(x):
+        return x + 1
+
+    assert increment(1).call() == 2
+    assert increment(increment(1)).call() == 3
+
+
 def test_aspect_nameless():
     @pfp.lazy
     def increment(x, inc=pfp.Aspect()):
@@ -35,6 +44,35 @@ def test_aspect_default():
     assert increment(1).call() == 4
 
 
+def test_input_context_detection():
+    @pfp.lazy
+    def increment(x, inc=pfp.Aspect(3)):
+        return x + inc
+
+    @pfp.lazy
+    def scale(x, slope=pfp.Aspect(2)):
+        return x*slope
+
+    assert 'inc' in increment(scale(1)).get_input_context()
+    assert 'slope' in increment(scale(1)).get_input_context()
+    assert 'unknown' not in increment(scale(1)).get_input_context()
+    assert increment(scale(1)).get_input_context(slope=4)['slope'] == 4
+    assert "value: 2" in str(increment(scale(1)).get_input_context())
+    assert "value: 3" in str(increment(scale(1)).get_input_context(slope=5))
+    assert "value: 5" in str(increment(scale(1)).get_input_context(slope=5))
+
+
+def test_aspect_default_operations():
+    @pfp.lazy
+    def strange(x, inc=pfp.Aspect(3)):
+        return x+inc
+
+    y1 = abs((strange(1)*2 + 1) / 3-6) == 3
+    assert y1.call()
+    with pytest.raises(Exception):
+        assert (not y1).call(inc=2)  # assert error check for not converting to bool
+
+
 def test_multi_call():
     @pfp.lazy
     def increment(x, inc=pfp.Aspect(1)):
@@ -66,15 +104,6 @@ def test_unused_imports():
         increment(increment(1), 2).call(inc=3)
 
 
-def test_unused_imports():
-    @pfp.lazy
-    def increment(x, inc=pfp.Aspect(1)):
-        return x + inc
-
-    with pytest.raises(Exception):
-        increment(increment(1)).call(incr=3)
-
-
 def test_priority_comparison():
     @pfp.lazy
     def increment(x, inc=pfp.Aspect(7, priority=pfp.Priority.INCREASED)):
@@ -291,6 +320,14 @@ def test_autoaspects():
     assert test_obj.call(scale=2.).apply().apply().x == 0.5
 
 
+def test_lazy_lists():
+    @pfp.lazy
+    @pfp.autoaspects
+    def zeros(length=10, default=0):
+        return [default] * length
+    assert zeros()[0](default=1) == 1
+
+
 def test_eager_autoaspect_cache():
     @pfp.eager
     @pfp.autoaspects
@@ -301,6 +338,25 @@ def test_eager_autoaspect_cache():
     assert id(zeros(10)) == id(zeros(10))
 
 
+def test_eager_no_cache():
+    @pfp.eager_no_cache
+    @pfp.autoaspects
+    def zeros(length=10):
+        return [0] * length
+
+    assert id(zeros(9)) != id(zeros(10))
+    assert id(zeros(10)) != id(zeros(10))
+
+
+def test_lazy_no_cache():
+    @pfp.lazy_no_cache
+    @pfp.autoaspects
+    def zeros(length=10):
+        return [0] * length
+    assert id(zeros()(length=9)) != id(zeros()(length=10))
+    assert id(zeros()(length=10)) != id(zeros()(length=10))
+
+
 def test_cache_scope():
     @pfp.eager
     @pfp.autoaspects
